@@ -96,8 +96,11 @@
 #' generalized boosted modeling as in *twang*; here, the number of trees is
 #' chosen based on cross-validation or out-of-bag error, rather than based on
 #' optimizing balance. \pkg{twang} should not be cited when using this method
-#' to estimate propensity scores. }
-#' \item{`"lasso"`, `"ridge"`, `"elasticnet"`}{ The propensity
+#' to estimate propensity scores. Note that because there is a random component to choosing the tuning
+#' parameter, results will vary across runs unless a [seed][set.seed] is
+#' set.}
+#' \item{`"lasso"`, `"ridge"`, `"elasticnet"`}{
+#'   The propensity
 #' scores are estimated using a lasso, ridge, or elastic net model,
 #' respectively. The `formula` supplied to `matchit()` is processed
 #' with [model.matrix()] and passed to \pkgfun{glmnet}{cv.glmnet}, and
@@ -128,7 +131,8 @@
 #' directly to \pkgfun{randomForest}{randomForest}, and
 #' \pkgfun{randomForest}{predict.randomForest} is used to compute the propensity
 #' scores. The `link` argument is ignored, and predicted probabilities are
-#' always returned as the distance measure.}
+#' always returned as the distance measure. Note that because there is a random component, results will vary across runs unless a [seed][set.seed] is
+#' set. }
 #' \item{`"nnet"`}{ The
 #' propensity scores are estimated using a single-hidden-layer neural network.
 #' The `formula` supplied to `matchit()` is passed directly to
@@ -156,35 +160,34 @@
 #' the linear predictor instead of the predicted probabilities. When
 #' `s.weights` is supplied to `matchit()`, it will not be passed to
 #' `bart2` because the `weights` argument in `bart2` does not
-#' correspond to sampling weights. }
+#' correspond to sampling weights. Note that because there is a random component to choosing the tuning
+#' parameter, results will vary across runs unless the `seed` argument is supplied to `distance.options`. Note that setting a seed using [set.seed()] is not sufficient to guarantee reproducibility unless single-threading is used. See \pkgfun{dbarts}{bart2} for details.}
 #' }
 #'
 #' ## Methods for computing distances from covariates
 #'
-#' The following methods involve computing a distance matrix from the covariates themselves
-#' without estimating a propensity score. Calipers on the distance measure and
-#' common support restrictions cannot be used, and the `distance`
-#' component of the output object will be empty because no propensity scores
-#' are estimated. The `link` and `distance.options` arguments are
-#' ignored with these methods. See the individual matching methods pages for
-#' whether these distances are allowed and how they are used. Each of these
-#' distance measures can also be calculated outside `matchit()` using its
-#' [corresponding function][euclidean_dist].
+#' The following methods involve computing a distance matrix from the covariates
+#' themselves without estimating a propensity score. Calipers on the distance
+#' measure and common support restrictions cannot be used, and the `distance`
+#' component of the output object will be empty because no propensity scores are
+#' estimated. The `link` and `distance.options` arguments are ignored with these
+#' methods. See the individual matching methods pages for whether these
+#' distances are allowed and how they are used. Each of these distance measures
+#' can also be calculated outside `matchit()` using its [corresponding function][euclidean_dist].
 #'
 #' \describe{
 #' \item{`"euclidean"`}{ The Euclidean distance is the raw
-#' distance between units, computed as \deqn{d_{ij} = \sqrt{(x_i - x_j)(x_i -
-#' x_j)'}} It is sensitive to the scale of the covariates, so covariates with
+#' distance between units, computed as \deqn{d_{ij} = \sqrt{(x_i - x_j)(x_i - x_j)'}} It is sensitive to the scale of the covariates, so covariates with
 #' larger scales will take higher priority. }
-#' \item{`"scaled_euclidean"`}{ The scaled Euclidean distance is the
+#' \item{`"scaled_euclidean"`}{
+#'  The scaled Euclidean distance is the
 #' Euclidean distance computed on the scaled (i.e., standardized) covariates.
 #' This ensures the covariates are on the same scale. The covariates are
 #' standardized using the pooled within-group standard deviations, computed by
 #' treatment group-mean centering each covariate before computing the standard
-#' deviation in the full sample. }
-#' \item{`"mahalanobis"`}{ The
-#' Mahalanobis distance is computed as \deqn{d_{ij} = \sqrt{(x_i -
-#' x_j)\Sigma^{-1}(x_i - x_j)'}} where \eqn{\Sigma} is the pooled within-group
+#' deviation in the full sample.
+#'  }
+#' \item{`"mahalanobis"`}{ The Mahalanobis distance is computed as \deqn{d_{ij} = \sqrt{(x_i - x_j)\Sigma^{-1}(x_i - x_j)'}} where \eqn{\Sigma} is the pooled within-group
 #' covariance matrix of the covariates, computed by treatment group-mean
 #' centering each covariate before computing the covariance in the full sample.
 #' This ensures the variables are on the same scale and accounts for the
@@ -197,63 +200,69 @@
 #' Mahalanobis distance but is not affinely invariant. }
 #' }
 #'
-#' To perform Mahalanobis distance matching *and* estimate propensity
-#' scores to be used for a purpose other than matching, the `mahvars`
-#' argument should be used along with a different specification to
-#' `distance`. See the individual matching method pages for details on how
-#' to use `mahvars`.
+#' To perform Mahalanobis distance matching *and* estimate propensity scores to
+#' be used for a purpose other than matching, the `mahvars` argument should be
+#' used along with a different specification to `distance`. See the individual
+#' matching method pages for details on how to use `mahvars`.
 #'
 #' ## Distances supplied as a numeric vector or matrix
 #'
-#' `distance` can also be supplied as a numeric vector whose values will be taken to
-#' function like propensity scores; their pairwise difference will define the
-#' distance between units. This might be useful for supplying propensity scores
-#' computed outside `matchit()` or resupplying `matchit()` with
-#' propensity scores estimated previously without having to recompute them.
+#' `distance` can also be supplied as a numeric vector whose values will be
+#' taken to function like propensity scores; their pairwise difference will
+#' define the distance between units. This might be useful for supplying
+#' propensity scores computed outside `matchit()` or resupplying `matchit()`
+#' with propensity scores estimated previously without having to recompute them.
 #'
 #' `distance` can also be supplied as a matrix whose values represent the
 #' pairwise distances between units. The matrix should either be a square, with
 #' a row and column for each unit (e.g., as the output of a call to
-#' `as.matrix(`[`dist`]`(.))`), or have as many rows as there are treated
-#' units and as many columns as there are control units (e.g., as the output of
-#' a call to [mahalanobis_dist()] or \pkgfun{optmatch}{match_on}). Distance values
-#' of `Inf` will disallow the corresponding units to be matched. When
-#' `distance` is a supplied as a numeric vector or matrix, `link` and
-#' `distance.options` are ignored.
+#' `as.matrix(`[`dist`]`(.))`), or have as many rows as there are treated units
+#' and as many columns as there are control units (e.g., as the output of a call
+#' to [mahalanobis_dist()] or \pkgfun{optmatch}{match_on}). Distance values of
+#' `Inf` will disallow the corresponding units to be matched. When `distance` is
+#' a supplied as a numeric vector or matrix, `link` and `distance.options` are
+#' ignored.
 #'
-#' @note
-#' In versions of *MatchIt* prior to 4.0.0, `distance` was
-#' specified in a slightly different way. When specifying arguments using the
-#' old syntax, they will automatically be converted to the corresponding method
-#' in the new syntax but a warning will be thrown. `distance = "logit"`,
-#' the old default, will still work in the new syntax, though `distance = "glm", link = "logit"` is preferred (note that these are the default
-#' settings and don't need to be made explicit).
+#' @note In versions of *MatchIt* prior to 4.0.0, `distance` was specified in a
+#' slightly different way. When specifying arguments using the old syntax, they
+#' will automatically be converted to the corresponding method in the new syntax
+#' but a warning will be thrown. `distance = "logit"`, the old default, will
+#' still work in the new syntax, though `distance = "glm", link = "logit"` is
+#' preferred (note that these are the default settings and don't need to be made
+#' explicit).
 #'
 #' @examples
 #' data("lalonde")
 #'
 #' # Linearized probit regression PS:
 #' m.out1 <- matchit(treat ~ age + educ + race + married +
-#'                     nodegree + re74 + re75, data = lalonde,
-#'                   distance = "glm", link = "linear.probit")
+#'                     nodegree + re74 + re75,
+#'                   data = lalonde,
+#'                   distance = "glm",
+#'                   link = "linear.probit")
 #' @examplesIf requireNamespace("mgcv", quietly = TRUE)
 #' # GAM logistic PS with smoothing splines (s()):
-#' m.out2 <- matchit(treat ~ s(age) + s(educ) + race + married +
-#'                     nodegree + re74 + re75, data = lalonde,
+#' m.out2 <- matchit(treat ~ s(age) + s(educ) +
+#'                     race + married +
+#'                     nodegree + re74 + re75,
+#'                   data = lalonde,
 #'                   distance = "gam")
 #' summary(m.out2$model)
 #' @examplesIf requireNamespace("CBPS", quietly = TRUE)
 #' # CBPS for ATC matching w/replacement, using the just-
 #' # identified version of CBPS (setting method = "exact"):
 #' m.out3 <- matchit(treat ~ age + educ + race + married +
-#'                     nodegree + re74 + re75, data = lalonde,
-#'                   distance = "cbps", estimand = "ATC",
+#'                     nodegree + re74 + re75,
+#'                   data = lalonde,
+#'                   distance = "cbps",
+#'                   estimand = "ATC",
 #'                   distance.options = list(method = "exact"),
 #'                   replace = TRUE)
 #' @examples
 #' # Mahalanobis distance matching - no PS estimated
 #' m.out4 <- matchit(treat ~ age + educ + race + married +
-#'                     nodegree + re74 + re75, data = lalonde,
+#'                     nodegree + re74 + re75,
+#'                   data = lalonde,
 #'                   distance = "mahalanobis")
 #'
 #' m.out4$distance #NULL
@@ -261,8 +270,10 @@
 #' # Mahalanobis distance matching with PS estimated
 #' # for use in a caliper; matching done on mahvars
 #' m.out5 <- matchit(treat ~ age + educ + race + married +
-#'                     nodegree + re74 + re75, data = lalonde,
-#'                   distance = "glm", caliper = .1,
+#'                     nodegree + re74 + re75,
+#'                   data = lalonde,
+#'                   distance = "glm",
+#'                   caliper = .1,
 #'                   mahvars = ~ age + educ + race + married +
 #'                                 nodegree + re74 + re75)
 #'
@@ -270,48 +281,56 @@
 #'
 #' # User-supplied propensity scores
 #' p.score <- fitted(glm(treat ~ age + educ + race + married +
-#'                         nodegree + re74 + re75, data = lalonde,
+#'                         nodegree + re74 + re75,
+#'                       data = lalonde,
 #'                       family = binomial))
 #'
 #' m.out6 <- matchit(treat ~ age + educ + race + married +
-#'                     nodegree + re74 + re75, data = lalonde,
+#'                     nodegree + re74 + re75,
+#'                   data = lalonde,
 #'                   distance = p.score)
 #'
-#' # User-supplied distance matrix using optmatch::match_on()
-#' @examplesIf requireNamespace("optmatch", quietly = TRUE)
-#' dist_mat <- optmatch::match_on(
+#' # User-supplied distance matrix using rank_mahalanobis()
+#' dist_mat <- robust_mahalanobis_dist(
 #'               treat ~ age + educ + race + nodegree +
-#'                 married + re74 + re75, data = lalonde,
-#'               method = "rank_mahalanobis")
+#'                 married + re74 + re75,
+#'               data = lalonde)
 #'
 #' m.out7 <- matchit(treat ~ age + educ + race + nodegree +
-#'                     married + re74 + re75, data = lalonde,
+#'                     married + re74 + re75,
+#'                   data = lalonde,
 #'                   distance = dist_mat)
-
 NULL
 
 #distance2glm-----------------
 distance2glm <- function(formula, data = NULL, link = "logit", ...) {
 
-  linear <- !is.null(link) && startsWith(as.character(link), "linear")
+  linear <- is_not_null(link) && startsWith(as.character(link), "linear")
   if (linear) link <- sub("linear.", "", as.character(link), fixed = TRUE)
 
-  A <- list(...)
-  A[!names(A) %in% c(names(formals(glm)), names(formals(glm.control)))] <- NULL
+  args <- unique(c(names(formals(glm)), names(formals(glm.control))))
+  A <- ...mget(args)
+  A[lengths(A) == 0L] <- NULL
 
-  res <- do.call("glm", c(list(formula = formula, data = data, family = quasibinomial(link = link)), A))
+  A[["data"]] <- data
+  A[["formula"]] <- formula
+  A[["family"]] <- {
+    if (is_null(A[["weights"]])) binomial(link = link)
+    else quasibinomial(link = link)
+  }
 
-  if (linear) pred <- predict(res, type = "link")
-  else pred <- predict(res, type = "response")
+  res <- do.call("glm", A)
 
-  return(list(model = res, distance = pred))
+  pred <- predict(res, type = if (linear) "link" else "response")
+
+  list(model = res, distance = pred)
 }
 
 #distance2gam-----------------
 distance2gam <- function(formula, data = NULL, link = "logit", ...) {
-  check.package("mgcv")
+  rlang::check_installed("mgcv")
 
-  linear <- !is.null(link) && startsWith(as.character(link), "linear")
+  linear <- is_not_null(link) && startsWith(as.character(link), "linear")
   if (linear) link <- sub("linear.", "", as.character(link), fixed = TRUE)
 
   A <- list(...)
@@ -322,97 +341,108 @@ distance2gam <- function(formula, data = NULL, link = "logit", ...) {
                                    weights = weights), A),
                  quote = TRUE)
 
-  if (linear) pred <- predict(res, type = "link")
-  else pred <- predict(res, type = "response")
+  pred <- predict(res, type = if (linear) "link" else "response")
 
-  return(list(model = res, distance = as.numeric(pred)))
+  list(model = res, distance = as.numeric(pred))
 }
 
 #distance2rpart-----------------
 distance2rpart <- function(formula, data = NULL, link = NULL, ...) {
-  check.package("rpart")
-  A <- list(...)
-  A[!names(A) %in% c(names(formals(rpart::rpart)), names(formals(rpart::rpart.control)))] <- NULL
+  rlang::check_installed("rpart")
+
+  args <- unique(c(names(formals(rpart::rpart)), names(formals(rpart::rpart.control))))
+  A <- ...mget(args)
+  A[lengths(A) == 0L] <- NULL
+
   A$formula <- formula
   A$data <- data
   A$method <- "class"
 
   res <- do.call(rpart::rpart, A)
-  return(list(model = res, distance = predict(res, type = "prob")[,"1"]))
+  list(model = res, distance = predict(res, type = "prob")[, "1"])
 }
 
 #distance2nnet-----------------
 distance2nnet <- function(formula, data = NULL, link = NULL, ...) {
-  check.package("nnet")
+  rlang::check_installed("nnet")
 
   A <- list(...)
   weights <- A$weights
   A$weights <- NULL
 
-  res <- do.call(nnet::nnet, c(list(formula, data, weights = weights, entropy = TRUE), A), quote = TRUE)
-  return(list(model = res, distance = drop(fitted(res))))
+  res <- do.call(nnet::nnet, c(list(formula, data = data, weights = weights, entropy = TRUE), A), quote = TRUE)
+  list(model = res, distance = drop(fitted(res)))
 }
 
 #distance2cbps-----------------
 distance2cbps <- function(formula, data = NULL, link = NULL, ...) {
-  check.package("CBPS")
+  rlang::check_installed("CBPS")
 
-  linear <- !is.null(link) && startsWith(as.character(link), "linear")
+  linear <- is_not_null(link) && startsWith(as.character(link), "linear")
 
   A <- list(...)
 
   A[["standardized"]] <- FALSE
-  if (is.null(A[["ATT"]])) {
-    if (is.null(A[["estimand"]])) A[["ATT"]] <- 1
+
+  if (is_null(A[["ATT"]])) {
+    if (is_null(A[["estimand"]])) {
+      A[["ATT"]] <- 1
+    }
     else {
       estimand <- toupper(A[["estimand"]])
       estimand <- match_arg(estimand, c("ATT", "ATC", "ATE"))
       A[["ATT"]] <- switch(estimand, "ATT" = 1, "ATC" = 2, 0)
     }
   }
-  if (is.null(A[["method"]])) {
+
+  if (is_null(A[["method"]])) {
     A[["method"]] <- if (isFALSE(A[["over"]])) "exact" else "over"
   }
+
   A[c("estimand", "over")] <- NULL
 
-  if (!is.null(A[["weights"]])) {
+  if (is_not_null(A[["weights"]])) {
     A[["sample.weights"]] <- A[["weights"]]
     A[["weights"]] <- NULL
   }
 
+  A[["formula"]] <- formula
+  A[["data"]] <- data
+
   capture.output({ #Keeps from printing message about treatment
-    res <- do.call(CBPS::CBPS, c(list(formula, data), A), quote = TRUE)
+    res <- do.call(CBPS::CBPS, A, quote = TRUE)
   })
 
   pred <- fitted(res)
   if (linear) pred <- qlogis(pred)
 
-  return(list(model = res, distance = pred))
+  list(model = res, distance = pred)
 }
 
 #distance2bart----------------
 distance2bart <- function(formula, data = NULL, link = NULL, ...) {
-  check.package("dbarts")
+  rlang::check_installed("dbarts")
 
-  linear <- !is.null(link) && startsWith(as.character(link), "linear")
+  linear <- is_not_null(link) && startsWith(as.character(link), "linear")
+
+  args <- unique(c(names(formals(dbarts::bart2)), names(formals(dbarts::dbartsControl))))
+  A <- ...mget(args)
+  A[lengths(A) == 0L] <- NULL
 
-  A <- list(...)
-  A[!names(A) %in% c(names(formals(dbarts::bart2)), names(formals(dbarts::dbartsControl)))] <- NULL
   A$formula <- formula
   A$data <- data
 
   res <- do.call(dbarts::bart2, A)
 
-  if (linear) pred <- fitted(res, type = "link")
-  else pred <- fitted(res, type = "response")
+  pred <- fitted(res, type = if (linear) "link" else "response")
 
-  return(list(model = res, distance = pred))
+  list(model = res, distance = pred)
 }
 
 # distance2bart <- function(formula, data, link = NULL, ...) {
-#   check.package("BART")
+#   rlang::check_installed("BART")
 #
-#   if (!is.null(link) && startsWith(as.character(link), "linear")) {
+#   if (is_not_null(link) && startsWith(as.character(link), "linear")) {
 #     linear <- TRUE
 #     link <- sub("linear.", "", as.character(link), fixed = TRUE)
 #   }
@@ -421,7 +451,7 @@ distance2bart <- function(formula, data = NULL, link = NULL, ...) {
 #   #Keep link probit because default in matchit is logit but probit is much faster with BART
 #   link <- "probit"
 #
-#   # if (is.null(link)) link <- "probit"
+#   # if (is_null(link)) link <- "probit"
 #   # else if (!link %in% c("probit", "logit")) {
 #   #   stop("'link' must be \"probit\" or \"logit\" with distance = \"bart\".", call. = FALSE)
 #   # }
@@ -436,7 +466,7 @@ distance2bart <- function(formula, data = NULL, link = NULL, ...) {
 #
 #   A <- list(...)
 #
-#   if (!is.null(A[["mc.cores"]]) && A[["mc.cores"]][1] > 1) fun <- BART::mc.gbart
+#   if (is_not_null(A[["mc.cores"]]) && A[["mc.cores"]][1] > 1) fun <- BART::mc.gbart
 #   else fun <- BART::gbart
 #
 #   res <- do.call(fun, c(list(X,
@@ -453,91 +483,116 @@ distance2bart <- function(formula, data = NULL, link = NULL, ...) {
 
 #distance2randomforest-----------------
 distance2randomforest <- function(formula, data = NULL, link = NULL, ...) {
-  check.package("randomForest")
+  rlang::check_installed("randomForest")
   newdata <- get_all_vars(formula, data)
-  treatvar <- as.character(formula[[2]])
+  treatvar <- as.character(formula[[2L]])
   newdata[[treatvar]] <- factor(newdata[[treatvar]], levels = c("0", "1"))
   res <- randomForest::randomForest(formula, data = newdata, ...)
-  return(list(model = res, distance = predict(res, type = "prob")[,"1"]))
+
+  list(model = res, distance = predict(res, type = "prob")[, "1"])
 }
 
 #distance2glmnet--------------
 distance2elasticnet <- function(formula, data = NULL, link = NULL, ...) {
-  check.package("glmnet")
+  rlang::check_installed("glmnet")
 
-  linear <- !is.null(link) && startsWith(as.character(link), "linear")
+  linear <- is_not_null(link) && startsWith(as.character(link), "linear")
   if (linear) link <- sub("linear.", "", as.character(link), fixed = TRUE)
 
-  A <- list(...)
-  s <- A[["s"]]
-  A[!names(A) %in% c(names(formals(glmnet::glmnet)), names(formals(glmnet::cv.glmnet)))] <- NULL
+  s <- ...get("s")
+  if (is_null(s)) {
+    s <- "lambda.1se"
+  }
 
-  if (is.null(link)) link <- "logit"
-  if (link == "logit") A$family <- "binomial"
-  else if (link == "log") A$family <- "poisson"
-  else A$family <- binomial(link = link)
+  args <- unique(c(names(formals(glmnet::glmnet)), names(formals(glmnet::cv.glmnet))))
+  A <- ...mget(args)
+  A[lengths(A) == 0L] <- NULL
 
-  if (is.null(A[["alpha"]])) A[["alpha"]] <- .5
+  if (is_null(link)) link <- "logit"
+
+  A$family <- switch(link,
+                     "logit" = "binomial",
+                     "log" = "poisson",
+                     binomial(link = link))
+
+  if (is_null(A[["alpha"]])) {
+    A[["alpha"]] <- .5
+  }
 
   mf <- model.frame(formula, data = data)
 
   A$y <- model.response(mf)
-  A$x <- model.matrix(update(formula, . ~ . + 1), mf)[,-1,drop = FALSE]
+  A$x <- model.matrix(update(formula, . ~ . + 1), mf)[, -1L, drop = FALSE]
 
   res <- do.call(glmnet::cv.glmnet, A)
 
-  if (is.null(s)) s <- "lambda.1se"
-
   pred <- drop(predict(res, newx = A$x, s = s,
-                  type = if (linear) "link" else "response"))
+                       type = if (linear) "link" else "response"))
 
-  return(list(model = res, distance = pred))
+  list(model = res, distance = pred)
 }
 distance2lasso <- function(formula, data = NULL, link = NULL, ...) {
-  A <- list(...)
-  A$alpha <- 1
-  do.call("distance2elasticnet", c(list(formula, data = data, link = link), A),
-          quote = TRUE)
+  if ("alpha" %in% ...names()) {
+    args <- unique(c("s", names(formals(glmnet::glmnet)), names(formals(glmnet::cv.glmnet))))
+    A <- ...mget(args)
+    A[lengths(A) == 0L] <- NULL
+
+    A$alpha <- 1
+    do.call("distance2elasticnet", c(list(formula, data = data, link = link), A),
+            quote = TRUE)
+  }
+  else {
+    distance2elasticnet(formula = formula, data = data, link = link, alpha = 1, ...)
+  }
 }
 distance2ridge <- function(formula, data = NULL, link = NULL, ...) {
-  A <- list(...)
-  A$alpha <- 0
-  do.call("distance2elasticnet", c(list(formula, data = data, link = link), A),
-          quote = TRUE)
+  if ("alpha" %in% ...names()) {
+    args <- unique(c("s", names(formals(glmnet::glmnet)), names(formals(glmnet::cv.glmnet))))
+    A <- ...mget(args)
+    A[lengths(A) == 0L] <- NULL
+
+    A$alpha <- 0
+    do.call("distance2elasticnet", c(list(formula, data = data, link = link), A),
+            quote = TRUE)
+  }
+  else {
+    distance2elasticnet(formula = formula, data = data, link = link, alpha = 0, ...)
+  }
 }
 
 #distance2gbm--------------
 distance2gbm <- function(formula, data = NULL, link = NULL, ...) {
-  check.package("gbm")
+  rlang::check_installed("gbm")
 
-  linear <- !is.null(link) && startsWith(as.character(link), "linear")
+  linear <- is_not_null(link) && startsWith(as.character(link), "linear")
 
-  A <- list(...)
+  method <- ...get("method")
 
-  method <- A[["method"]]
-  A[!names(A) %in% names(formals(gbm::gbm))] <- NULL
+  args <- names(formals(gbm::gbm))
+  A <- ...mget(args)
+  A[lengths(A) == 0L] <- NULL
 
   A$formula <- formula
   A$data <- data
   A$distribution <- "bernoulli"
 
-  if (is.null(A[["n.trees"]])) A[["n.trees"]] <- 1e4
-  if (is.null(A[["interaction.depth"]])) A[["interaction.depth"]] <- 3
-  if (is.null(A[["shrinkage"]])) A[["shrinkage"]] <- .01
-  if (is.null(A[["bag.fraction"]])) A[["bag.fraction"]] <- 1
-  if (is.null(A[["cv.folds"]])) A[["cv.folds"]] <- 5
-  if (is.null(A[["keep.data"]])) A[["keep.data"]] <- FALSE
+  if (is_null(A[["n.trees"]])) A[["n.trees"]] <- 1e4
+  if (is_null(A[["interaction.depth"]])) A[["interaction.depth"]] <- 3
+  if (is_null(A[["shrinkage"]])) A[["shrinkage"]] <- .01
+  if (is_null(A[["bag.fraction"]])) A[["bag.fraction"]] <- 1
+  if (is_null(A[["cv.folds"]])) A[["cv.folds"]] <- 5
+  if (is_null(A[["keep.data"]])) A[["keep.data"]] <- FALSE
 
   if (A[["cv.folds"]] <= 1 && A[["bag.fraction"]] == 1) {
-    stop("Either `bag.fraction` must be less than 1 or `cv.folds` must be greater than 1 when using `distance = \"gbm\"`.",
-         call. = FALSE)
+    .err('either `bag.fraction` must be less than 1 or `cv.folds` must be greater than 1 when using `distance = "gbm"`')
   }
-  if (is.null(method)) {
+
+  if (is_null(method)) {
     if (A[["bag.fraction"]] < 1) method <- "OOB"
     else method <- "cv"
   }
   else if (!tolower(method) %in% c("oob", "cv")) {
-    stop("`distance.options$method` should be one of \"OOB\" or \"cv\".", call. = FALSE)
+    .err('`distance.options$method` should be one of "OOB" or "cv"')
   }
 
   res <- do.call(gbm::gbm, A)
@@ -547,5 +602,5 @@ distance2gbm <- function(formula, data = NULL, link = NULL, ...) {
   pred <- drop(predict(res, newdata = data, n.trees = best.tree,
                        type = if (linear) "link" else "response"))
 
-  return(list(model = res, distance = pred))
+  list(model = res, distance = pred)
 }
