@@ -72,7 +72,7 @@
 #' place when `distance` corresponds to a propensity score (e.g., for
 #' caliper matching or to discard units for common support). If specified, the
 #' distance measure will not be used in matching.
-#' @param antiexact for which variables ant-exact matching should take place.
+#' @param antiexact for which variables anti-exact matching should take place.
 #' Anti-exact matching is processed using \pkgfun{optmatch}{antiExactMatch}.
 #' @param discard a string containing a method for discarding units outside a
 #' region of common support. Only allowed when `distance` corresponds to a
@@ -82,7 +82,7 @@
 #' @param s.weights the variable containing sampling weights to be incorporated
 #' into propensity score models and balance statistics.
 #' @param caliper the width(s) of the caliper(s) used for caliper matching.
-#' Calipers are processed by \pkgfun{optmatch}{caliper}. See Notes and Examples.
+#' Calipers are processed by \pkgfun{optmatch}{caliper}. Positive and negative calipers are allowed. See Notes and Examples.
 #' @param std.caliper `logical`; when calipers are specified, whether they
 #' are in standard deviation units (`TRUE`) or raw units (`FALSE`).
 #' @param verbose `logical`; whether information about the matching
@@ -197,15 +197,18 @@
 #'
 #' # Optimal full PS matching
 #' m.out1 <- matchit(treat ~ age + educ + race + nodegree +
-#'                     married + re74 + re75, data = lalonde,
+#'                     married + re74 + re75,
+#'                   data = lalonde,
 #'                   method = "full")
 #' m.out1
 #' summary(m.out1)
 #'
 #' # Optimal full Mahalanobis distance matching within a PS caliper
 #' m.out2 <- matchit(treat ~ age + educ + race + nodegree +
-#'                     married + re74 + re75, data = lalonde,
-#'                   method = "full", caliper = .01,
+#'                     married + re74 + re75,
+#'                   data = lalonde,
+#'                   method = "full",
+#'                   caliper = .01,
 #'                   mahvars = ~ age + educ + re74 + re75)
 #' m.out2
 #' summary(m.out2, un = FALSE)
@@ -213,13 +216,17 @@
 #' # Optimal full Mahalanobis distance matching within calipers
 #' # of 500 on re74 and re75
 #' m.out3 <- matchit(treat ~ age + educ + re74 + re75,
-#'                   data = lalonde, distance = "mahalanobis",
+#'                   data = lalonde,
+#'                   distance = "mahalanobis",
 #'                   method = "full",
-#'                   caliper = c(re74 = 500, re75 = 500),
+#'                   caliper = c(re74 = 500,
+#'                               re75 = 500),
 #'                   std.caliper = FALSE)
 #' m.out3
-#' summary(m.out3, addlvariables = ~race + nodegree + married,
-#'         data = lalonde, un = FALSE)
+#' summary(m.out3,
+#'         addlvariables = ~race + nodegree + married,
+#'         data = lalonde,
+#'         un = FALSE)
 NULL
 
 matchit2full <- function(treat, formula, data, distance, discarded,
@@ -230,17 +237,11 @@ matchit2full <- function(treat, formula, data, distance, discarded,
 
   rlang::check_installed("optmatch")
 
-  if (verbose) cat("Full matching... \n")
+  .cat_verbose("Full matching... \n", verbose = verbose)
 
-  A <- list(...)
-
-  fm.args <- c("omit.fraction", "mean.controls", "tol", "solver")
-  A[!names(A) %in% fm.args] <- NULL
-
-  #Set max problem size to Inf and return to original value after match
-  omps <- getOption("optmatch_max_problem_size")
-  on.exit(options(optmatch_max_problem_size = omps))
-  options(optmatch_max_problem_size = Inf)
+  .args <- c("omit.fraction", "mean.controls", "tol", "solver")
+  A <- ...mget(.args)
+  A[lengths(A) == 0L] <- NULL
 
   estimand <- toupper(estimand)
   estimand <- match_arg(estimand, c("ATT", "ATC", "ATE"))
@@ -255,10 +256,10 @@ matchit2full <- function(treat, formula, data, distance, discarded,
 
   treat_ <- setNames(as.integer(treat[!discarded] == focal), names(treat)[!discarded])
 
-  # if (!is.null(data)) data <- data[!discarded,]
+  # if (is_not_null(data)) data <- data[!discarded,]
 
   if (is.full.mahalanobis) {
-    if (length(attr(terms(formula, data = data), "term.labels")) == 0) {
+    if (is_null(attr(terms(formula, data = data), "term.labels"))) {
       .err(sprintf("covariates must be specified in the input formula when `distance = \"%s\"`",
                    attr(is.full.mahalanobis, "transform")))
     }
@@ -269,20 +270,24 @@ matchit2full <- function(treat, formula, data, distance, discarded,
   max.controls <- attr(ratio, "max.controls")
 
   #Exact matching strata
-  if (!is.null(exact)) {
+  if (is_not_null(exact)) {
     ex <- factor(exactify(model.frame(exact, data = data),
                           sep = ", ", include_vars = TRUE)[!discarded])
-    cc <- intersect(as.integer(ex)[treat_==1], as.integer(ex)[treat_==0])
-    if (length(cc) == 0) .err("No matches were found")
+
+    cc <- Reduce("intersect", lapply(unique(treat_), function(t) unclass(ex)[treat_ == t]))
+
+    if (is_null(cc)) {
+      .err("No matches were found")
+    }
   }
   else {
-    ex <- factor(rep("_", length(treat_)), levels = "_")
+    ex <- gl(1, length(treat_), labels = "_")
     cc <- 1
   }
 
   #Create distance matrix; note that Mahalanobis distance computed using entire
   #sample (minus discarded), like method2nearest, as opposed to within exact strata, like optmatch.
-  if (!is.null(mahvars)) {
+  if (is_not_null(mahvars)) {
     transform <- if (is.full.mahalanobis) attr(is.full.mahalanobis, "transform") else "mahalanobis"
     mahcovs <- transform_covariates(mahvars, data = data, method = transform,
                                     s.weights = s.weights, treat = treat,
@@ -297,17 +302,19 @@ matchit2full <- function(treat, formula, data, distance, discarded,
   }
 
   #Transpose distance mat as needed
-  if (focal == 0) mo <- t(mo)
+  if (focal == 0) {
+    mo <- t(mo)
+  }
 
   #Remove discarded units from distance mat
   mo <- mo[!discarded[treat == focal], !discarded[treat != focal], drop = FALSE]
   dimnames(mo) <- list(names(treat_)[treat_ == 1], names(treat_)[treat_ == 0])
 
-  mo <- optmatch::match_on(mo, data = data[!discarded,, drop = FALSE])
+  mo <- optmatch::match_on(mo, data = as.data.frame(data)[!discarded, , drop = FALSE])
   mo <- optmatch::as.InfinitySparseMatrix(mo)
 
   #Process antiexact
-  if (!is.null(antiexact)) {
+  if (is_not_null(antiexact)) {
     antiexactcovs <- model.frame(antiexact, data)
     for (i in seq_len(ncol(antiexactcovs))) {
       mo <- mo + optmatch::antiExactMatch(antiexactcovs[[i]][!discarded], z = treat_)
@@ -315,67 +322,81 @@ matchit2full <- function(treat, formula, data, distance, discarded,
   }
 
   #Process caliper
-  if (!is.null(caliper)) {
+  if (is_not_null(caliper)) {
     if (min.controls != 0) {
       .err("calipers cannot be used with `method = \"full\"` when `min.controls` is specified")
     }
 
-    if (any(names(caliper) != "")) {
+    if (any(nzchar(names(caliper)))) {
       cov.cals <- setdiff(names(caliper), "")
-      calcovs <- get.covs.matrix(reformulate(cov.cals, intercept = FALSE), data = data)
+      calcovs <- get_covs_matrix(reformulate(cov.cals, intercept = FALSE), data = data)
     }
+
     for (i in seq_along(caliper)) {
-      if (names(caliper)[i] != "") {
+      if (nzchar(names(caliper)[i])) {
         mo_cal <- optmatch::match_on(setNames(calcovs[!discarded, names(caliper)[i]], names(treat_)), z = treat_)
       }
-      else if (is.null(mahvars) || is.matrix(distance)) {
+      else if (is_null(mahvars) || is.matrix(distance)) {
         mo_cal <- mo
       }
       else {
         mo_cal <- optmatch::match_on(setNames(distance[!discarded], names(treat_)), z = treat_)
       }
 
-      mo <- mo + optmatch::caliper(mo_cal, caliper[i])
+      mo <- mo + optmatch::caliper(mo_cal, abs(caliper[i]),
+                                   compare = if (caliper[i] >= 0) `<=` else `>=`)
     }
+
     rm(mo_cal)
   }
 
   #Initialize pair membership; must include names
-  pair <- setNames(rep(NA_character_, length(treat)), names(treat))
+  pair <- rep_with(NA_character_, treat)
   p <- setNames(vector("list", nlevels(ex)), levels(ex))
 
-  t_df <- data.frame(treat)
+  A$data <- data.frame(treat) #just to get rownames; not actually used in matching
+  A$min.controls <- min.controls
+  A$max.controls <- max.controls
 
   for (e in levels(ex)[cc]) {
-    if (nlevels(ex) > 1) {
-      if (verbose) {
-        cat(sprintf("Matching subgroup %s/%s: %s...\n",
-                    match(e, levels(ex)[cc]), length(cc), e))
-      }
-      mo_ <- mo[ex[treat_==1] == e, ex[treat_==0] == e]
-    }
-    else mo_ <- mo
+    if (nlevels(ex) > 1L) {
+      .cat_verbose(sprintf("Matching subgroup %s/%s: %s...\n",
+                           match(e, levels(ex)[cc]), length(cc), e),
+                   verbose = verbose)
 
-    if (any(dim(mo_) == 0) || !any(is.finite(mo_))) next
-    else if (all(dim(mo_) == 1) && all(is.finite(mo_))) {
+      mo_ <- mo[ex[treat_ == 1] == e, ex[treat_ == 0] == e]
+    }
+    else {
+      mo_ <- mo
+    }
+
+    if (any(dim(mo_) == 0) || !any(is.finite(mo_))) {
+      next
+    }
+
+    if (all_equal_to(dim(mo_), 1) && all(is.finite(mo_))) {
       pair[ex == e] <- paste(1, e, sep = "|")
       next
     }
 
-    matchit_try({
-      p[[e]] <- do.call(optmatch::fullmatch,
-                        c(list(mo_,
-                               min.controls = min.controls,
-                               max.controls = max.controls,
-                               data = t_df), #just to get rownames; not actually used in matching
-                          A))
-    }, from = "optmatch")
+    A$x <- mo_
+
+    rlang::with_options({
+      matchit_try({
+        p[[e]] <- do.call(optmatch::fullmatch, A)
+      }, from = "optmatch")
+    }, optmatch_max_problem_size = Inf)
 
     pair[names(p[[e]])[!is.na(p[[e]])]] <- paste(as.character(p[[e]][!is.na(p[[e]])]), e, sep = "|")
   }
 
-  if (all(is.na(pair))) .err("No matches were found")
-  if (length(p) == 1) p <- p[[1]]
+  if (all(is.na(pair))) {
+    .err("No matches were found")
+  }
+
+  if (length(p) == 1L) {
+    p <- p[[1]]
+  }
 
   psclass <- factor(pair)
   levels(psclass) <- seq_len(nlevels(psclass))
@@ -384,14 +405,15 @@ matchit2full <- function(treat, formula, data, distance, discarded,
   #No match.matrix because treated units don't index matched strata (i.e., more than one
   #treated unit can be in the same stratum). Stratum information is contained in subclass.
 
-  if (verbose) cat("Calculating matching weights... ")
+  .cat_verbose("Calculating matching weights... ",
+               verbose = verbose)
 
   res <- list(subclass = psclass,
               weights = get_weights_from_subclass(psclass, treat, estimand),
               obj = p)
 
-  if (verbose) cat("Done.\n")
+  .cat_verbose("Done.\n", verbose = verbose)
 
-  class(res) <- c("matchit")
+  class(res) <- "matchit"
   res
 }
