@@ -80,7 +80,7 @@
 #' to the distance matrix. Use `mahvars` to only supply a subset. Even if
 #' `mahvars` is specified, balance will be optimized on all covariates in
 #' `formula`. See Details.
-#' @param antiexact for which variables ant-exact matching should take place.
+#' @param antiexact for which variables anti-exact matching should take place.
 #' Anti-exact matching is processed using the `restrict` argument to
 #' `Matching::GenMatch()` and `Matching::Match()`.
 #' @param discard a string containing a method for discarding units outside a
@@ -190,6 +190,10 @@
 #' support the ATE as an estimand, `matchit()` only supports the ATT and
 #' ATC for genetic matching.
 #'
+#' ## Reproducibility
+#'
+#' Genetic matching involves a random component, so a seed must be set using [set.seed()] to ensure reproducibility. When `cluster` is used for parallel processing, the seed must be compatible with parallel processing (e.g., by setting `kind = "L'Ecuyer-CMRG"`).
+#'
 #' @seealso [matchit()] for a detailed explanation of the inputs and outputs of
 #' a call to `matchit()`.
 #'
@@ -217,7 +221,8 @@
 #'
 #' # 1:1 genetic matching with PS as a covariate
 #' m.out1 <- matchit(treat ~ age + educ + race + nodegree +
-#'                     married + re74 + re75, data = lalonde,
+#'                     married + re74 + re75,
+#'                   data = lalonde,
 #'                   method = "genetic",
 #'                   pop.size = 10) #use much larger pop.size
 #' m.out1
@@ -225,9 +230,12 @@
 #'
 #' # 2:1 genetic matching with replacement without PS
 #' m.out2 <- matchit(treat ~ age + educ + race + nodegree +
-#'                     married + re74 + re75, data = lalonde,
-#'                   method = "genetic", replace = TRUE,
-#'                   ratio = 2, distance = "mahalanobis",
+#'                     married + re74 + re75,
+#'                   data = lalonde,
+#'                   method = "genetic",
+#'                   replace = TRUE,
+#'                   ratio = 2,
+#'                   distance = "mahalanobis",
 #'                   pop.size = 10) #use much larger pop.size
 #' m.out2
 #' summary(m.out2, un = FALSE)
@@ -236,7 +244,8 @@
 #' # within calipers on PS and educ; other variables are
 #' # used to estimate PS
 #' m.out3 <- matchit(treat ~ age + educ + race + nodegree +
-#'                     married + re74 + re75, data = lalonde,
+#'                     married + re74 + re75,
+#'                   data = lalonde,
 #'                   method = "genetic",
 #'                   mahvars = ~ age + educ + re74 + re75,
 #'                   caliper = c(.05, educ = 2),
@@ -255,9 +264,11 @@ matchit2genetic <- function(treat, data, distance, discarded,
 
   rlang::check_installed(c("Matching", "rgenoud"))
 
-  if (verbose) cat("Genetic matching... \n")
+  .cat_verbose("Genetic matching...\n", verbose = verbose)
 
-  A <- list(...)
+  .args <- names(formals(Matching::GenMatch))
+  A <- ...mget(.args)
+  A[lengths(A) == 0L] <- NULL
 
   estimand <- toupper(estimand)
   estimand <- match_arg(estimand, c("ATT", "ATC"))
@@ -273,11 +284,11 @@ matchit2genetic <- function(treat, data, distance, discarded,
   if (!replace) {
     if (sum(!discarded & treat != focal) < sum(!discarded & treat == focal)) {
       .wrn(sprintf("fewer %s units than %s units; not all %s units will get a match",
-                      tc[2], tc[1], tc[1]))
+                   tc[2L], tc[1L], tc[1L]))
     }
-    else if (sum(!discarded & treat != focal) < sum(!discarded & treat == focal)*ratio) {
+    else if (sum(!discarded & treat != focal) < sum(!discarded & treat == focal) * ratio) {
       .err(sprintf("not enough %s units for %s matches for each %s unit",
-                   tc[2], ratio, tc[1]))
+                   tc[2L], ratio, tc[1L]))
     }
   }
 
@@ -286,13 +297,12 @@ matchit2genetic <- function(treat, data, distance, discarded,
   n.obs <- length(treat)
   n1 <- sum(treat == 1)
 
-  if (is.null(names(treat))) names(treat) <- seq_len(n.obs)
+  if (is_null(names(treat))) names(treat) <- seq_len(n.obs)
 
   m.order <- {
-    if (is.null(distance)) match_arg(m.order, c("data", "random"))
-    else if (!is.null(m.order)) match_arg(m.order, c("largest", "smallest", "random", "data"))
-    else if (estimand == "ATC") "smallest"
-    else "largest"
+    if (is_null(distance)) match_arg(m.order, c("data", "random"))
+    else if (is_null(m.order)) switch(estimand, "ATC" = "smallest", "largest")
+    else match_arg(m.order, c("largest", "smallest", "data", "random"))
   }
 
   ord <- switch(m.order,
@@ -300,127 +310,191 @@ matchit2genetic <- function(treat, data, distance, discarded,
                 "smallest" = order(distance),
                 "random" = sample.int(n.obs),
                 "data" = seq_len(n.obs))
-  ord <- ord[!ord %in% which(discarded)]
+
+  if (any(discarded)) {
+    ord <- ord[!ord %in% which(discarded)]
+  }
 
   #Create X (matching variables) and covs_to_balance
-  covs_to_balance <- get.covs.matrix(formula, data = data)
-  if (!is.null(mahvars)) {
-    X <- get.covs.matrix.for.dist(mahvars, data = data)
-  }
-  else if (is.full.mahalanobis) {
-    X <- covs_to_balance
-  }
-  else {
-    X <- cbind(covs_to_balance, distance)
-  }
+  covs_to_balance <- get_covs_matrix(formula, data = data)
 
-  if (ncol(covs_to_balance) == 0) {
+  if (ncol(covs_to_balance) == 0L) {
     .err("covariates must be specified in the input formula to use genetic matching")
   }
 
-  #Process exact; exact.log will be supplied to GenMatch() and Match()
-  if (!is.null(exact)) {
-    #Add covariates in exact not in X to X
-    ex <- as.integer(factor(exactify(model.frame(exact, data = data), names(treat), sep = ", ", include_vars = TRUE)))
+  X <- {
+    if (is_not_null(mahvars)) get_covs_matrix_for_dist(mahvars, data = data)
+    else if (is.full.mahalanobis) covs_to_balance
+    else cbind(covs_to_balance, distance)
+  }
 
-    cc <- intersect(ex[treat==1], ex[treat==0])
-    if (length(cc) == 0) .err("No matches were found")
+  #Process exact; exact.log will be supplied to GenMatch() and Match()
+  if (is_not_null(exact)) {
+    #Add covariates in exact not in X to X
+    ex <- unclass(exactify(model.frame(exact, data = data), names(treat),
+                           sep = ", ", include_vars = TRUE))
+
+    cc <- intersect(ex[treat == 1], ex[treat == 0])
+
+    if (is_null(cc)) {
+      .err("No matches were found")
+    }
 
     X <- cbind(X, ex)
 
-    exact.log <- c(rep(FALSE, ncol(X) - 1), TRUE)
-  }
-  else exact.log <- ex <- NULL
-
-  #Process caliper; cal will be supplied to GenMatch() and Match()
-  if (!is.null(caliper)) {
-    #Add covariates in caliper other than distance (cov.cals) not in X to X
-    cov.cals <- setdiff(names(caliper), "")
-    if (length(cov.cals) > 0 && any(!cov.cals %in% colnames(X))) {
-      calcovs <- get.covs.matrix(reformulate(cov.cals[!cov.cals %in% colnames(X)]), data = data)
-      X <- cbind(X, calcovs)
-
-      #Expand exact.log for newly added covariates
-      if (!is.null(exact.log)) exact.log <- c(exact.log, rep(FALSE, ncol(calcovs)))
-    }
-    else cov.cals <- NULL
-
-    #Matching::Match multiplies calipers by pop SD, so we need to divide by pop SD to unstandardize
-    pop.sd <- function(x) sqrt(sum((x-mean(x))^2)/length(x))
-    caliper <- caliper / vapply(names(caliper), function(x) {
-      if (x == "") pop.sd(distance[!discarded])
-      else pop.sd(X[!discarded, x])
-    }, numeric(1L))
-
-    #cal needs one value per variable in X
-    cal <- setNames(rep(Inf, ncol(X)), colnames(X))
-
-    #First put covariate calipers into cal
-    if (length(cov.cals) > 0) {
-      cal[intersect(cov.cals, names(cal))] <- caliper[intersect(cov.cals, names(cal))]
-    }
-
-    #Then put distance caliper into cal
-    if ("" %in% names(caliper)) {
-      dist.cal <- caliper[names(caliper) == ""]
-      if (!is.null(mahvars)) {
-        #If mahvars specified, distance is not yet in X, so add it to X
-        X <- cbind(X, distance)
-        cal <- c(cal, dist.cal)
-
-        #Expand exact.log for newly added distance
-        if (!is.null(exact.log)) exact.log <- c(exact.log, FALSE)
-      }
-      else {
-        #Otherwise, distance is in X at the specified index
-        cal[ncol(covs_to_balance) + 1] <- dist.cal
-      }
-    }
-    else dist.cal <- NULL
-
+    exact.log <- c(rep.int(FALSE, ncol(X) - 1L), TRUE)
   }
   else {
-    cal <- dist.cal <- cov.cals <- NULL
+    exact.log <- ex <- NULL
   }
 
   #Reorder data according to m.order since Match matches in order of data;
   #ord already excludes discarded units
 
   treat_ <- treat[ord]
-  covs_to_balance <- covs_to_balance[ord,,drop = FALSE]
-  X <- X[ord,,drop = FALSE]
-  if (!is.null(s.weights)) s.weights <- s.weights[ord]
+  covs_to_balance <- covs_to_balance[ord, , drop = FALSE]
+  X <- X[ord, , drop = FALSE]
+  if (is_not_null(s.weights)) s.weights <- s.weights[ord]
 
-  if (!is.null(antiexact)) {
-    antiexactcovs <- model.frame(antiexact, data)[ord,,drop = FALSE]
-    antiexact_restrict <- cbind(do.call("rbind", lapply(seq_len(ncol(antiexactcovs)), function(i) {
-      unique.vals <- unique(antiexactcovs[,i])
-      do.call("rbind", lapply(unique.vals, function(u) {
-        t(combn(which(antiexactcovs[,i] == u), 2))
+  #Process caliper; cal will be supplied to GenMatch() and Match()
+  cal <- dist.cal <- cov.cals <- NULL
+  if (is_not_null(caliper) && any(caliper < 0)) {
+    neg.cal <- names(caliper)[caliper < 0]
+
+    if (any(nzchar(neg.cal))) {
+      negcalcovs <- get_covs_matrix(reformulate(neg.cal[nzchar(neg.cal)]), data = data)[ord, , drop = FALSE]
+      negcalcovs_restrict <- do.call("rbind", lapply(seq_len(ncol(negcalcovs)), function(i) {
+        do.call("rbind", lapply(which(treat_ == 1), function(j) {
+          restricted_controls <- which(treat_ == 0 & abs(negcalcovs[j, i] - negcalcovs[, i]) <= -caliper[neg.cal[nzchar(neg.cal)][i]])
+
+          if (is_null(restricted_controls)) {
+            return(NULL)
+          }
+
+          cbind(j, restricted_controls, -1)
+        }))
       }))
-    })), -1)
-    if (!is.null(A[["restrict"]])) A[["restrict"]] <- rbind(A[["restrict"]], antiexact_restrict)
-    else A[["restrict"]] <- antiexact_restrict
+
+      if (is_not_null(negcalcovs_restrict)) {
+        A[["restrict"]] <- {
+          if (is_null(A[["restrict"]])) unique(negcalcovs_restrict)
+          else rbind(A[["restrict"]], unique(negcalcovs_restrict))
+        }
+      }
+    }
+
+    if (!all(nzchar(neg.cal))) {
+      negcaldist_restrict <- do.call("rbind", lapply(which(treat_ == 1), function(j) {
+        restricted_controls <- which(treat_ == 0 & abs(distance[ord][j] - distance[ord]) <= -caliper[!nzchar(names(caliper))])
+
+        if (is_null(restricted_controls)) {
+          return(NULL)
+        }
+
+        cbind(j, restricted_controls, -1)
+      }))
+
+      if (is_not_null(negcaldist_restrict)) {
+        A[["restrict"]] <- {
+          if (is_null(A[["restrict"]])) unique(negcaldist_restrict)
+          else rbind(A[["restrict"]], unique(negcaldist_restrict))
+        }
+      }
+    }
+
+    caliper <- caliper[caliper >= 0]
+  }
+
+  #Add covariates in caliper other than distance (cov.cals) not in X to X
+  if (is_not_null(caliper)) {
+    cov.cals <- setdiff(names(caliper), "")
+    if (is_not_null(cov.cals) && !all(cov.cals %in% colnames(X))) {
+      calcovs <- get_covs_matrix(reformulate(cov.cals[!cov.cals %in% colnames(X)]), data = data)[ord, , drop = FALSE]
+      X <- cbind(X, calcovs)
+
+      #Expand exact.log for newly added covariates
+      if (is_not_null(exact.log)) {
+        exact.log <- c(exact.log, rep.int(FALSE, ncol(calcovs)))
+      }
+    }
+
+    #Matching::Match multiplies calipers by pop SD, so we need to divide by pop SD to unstandardize
+    pop.sd <- function(x) sqrt(sum((x - mean(x))^2) / length(x))
+    caliper <- caliper / vapply(names(caliper), function(x) {
+      if (x == "") pop.sd(distance[ord])
+      else pop.sd(X[, x])
+    }, numeric(1L))
+
+    #cal needs one value per variable in X
+    cal <- setNames(rep.int(Inf, ncol(X)), colnames(X))
+
+    #First put covariate calipers into cal
+    if (is_not_null(cov.cals)) {
+      cal[intersect(cov.cals, names(cal))] <- caliper[intersect(cov.cals, names(cal))]
+    }
+
+    #Then put distance caliper into cal
+    if (!all(nzchar(names(caliper)))) {
+      dist.cal <- caliper[!nzchar(names(caliper))]
+      if (is_not_null(mahvars)) {
+        #If mahvars specified, distance is not yet in X, so add it to X
+        X <- cbind(X, distance[ord])
+        cal <- c(cal, dist.cal)
+
+        #Expand exact.log for newly added distance
+        if (is_not_null(exact.log)) exact.log <- c(exact.log, FALSE)
+      }
+      else {
+        #Otherwise, distance is in X at the specified index
+        cal[ncol(covs_to_balance) + 1L] <- dist.cal
+      }
+    }
+    else {
+      dist.cal <- NULL
+    }
+  }
+
+  if (is_not_null(antiexact)) {
+    antiexactcovs <- model.frame(antiexact, data)[ord, , drop = FALSE]
+    antiexact_restrict <- do.call("rbind", lapply(seq_len(ncol(antiexactcovs)), function(i) {
+      do.call("rbind", lapply(which(treat_ == 1), function(j) {
+        restricted_controls <- which(treat_ == 0 & antiexactcovs[[i]][j] == antiexactcovs[[i]])
+
+        if (is_null(restricted_controls)) {
+          return(NULL)
+        }
+
+        cbind(j, restricted_controls, -1)
+      }))
+    }))
+
+    if (is_not_null(antiexact_restrict)) {
+      A[["restrict"]] <- {
+        if (is_null(A[["restrict"]])) unique(antiexact_restrict)
+        else rbind(A[["restrict"]], unique(antiexact_restrict))
+      }
+    }
   }
   else {
     antiexactcovs <- NULL
   }
 
-  if (is.null(A[["distance.tolerance"]])) {
+  if (is_null(A[["distance.tolerance"]])) {
     A[["distance.tolerance"]] <- 0
   }
 
   if (use.genetic) {
-    matchit_try({
-      g.out <- do.call(Matching::GenMatch,
-                       c(list(Tr = treat_, X = X, BalanceMatrix = covs_to_balance,
-                              M = ratio, exact = exact.log, caliper = cal,
-                              replace = replace, estimand = "ATT", ties = FALSE,
-                              CommonSupport = FALSE, verbose = verbose,
-                              weights = s.weights, print.level = 2*verbose),
-                         A[names(A) %in% names(formals(Matching::GenMatch))]))
-    }, from = "Matching",
-    dont_warn_if = "replace==FALSE, but there are more (weighted) treated obs than control obs.")
+      matchit_try({
+        g.out <- do.call(Matching::GenMatch,
+                         c(list(Tr = treat_, X = X, BalanceMatrix = covs_to_balance,
+                                M = ratio, exact = exact.log, caliper = cal,
+                                replace = replace, estimand = "ATT", ties = FALSE,
+                                CommonSupport = FALSE, verbose = verbose,
+                                weights = s.weights, print.level = 2 * verbose),
+                           A[names(A) %in% .args]))
+      }, from = "Matching",
+      dont_warn_if = c("replace==FALSE, but there are more (weighted) treated obs than control obs",
+                       "no valid matches"))
   }
   else {
     #For debugging
@@ -434,19 +508,25 @@ matchit2genetic <- function(treat, data, distance, discarded,
 
   ind_ <- seq_along(treat)[ord]
 
-  # if (!isFALSE(A$use.Match)) {
   matchit_try({
     m.out <- Matching::Match(Tr = treat_, X = X,
                              M = ratio, exact = exact.log, caliper = cal,
                              replace = replace, estimand = "ATT", ties = FALSE,
                              weights = s.weights, CommonSupport = FALSE,
                              distance.tolerance = A[["distance.tolerance"]], Weight = 3,
-                             Weight.matrix = if (use.genetic) g.out
-                             else if (is.null(s.weights)) generalized_inverse(cor(X))
-                             else generalized_inverse(cov.wt(X, s.weights, cor = TRUE)$cor),
+                             Weight.matrix = {
+                               if (use.genetic) g.out
+                               else if (is_null(s.weights)) generalized_inverse(cor(X))
+                               else generalized_inverse(cov.wt(X, s.weights, cor = TRUE)$cor)
+                             },
                              restrict = A[["restrict"]], version = "fast")
   }, from = "Matching",
-  dont_warn_if = "replace==FALSE, but there are more (weighted) treated obs than control obs.")
+  dont_warn_if = c("replace==FALSE, but there are more (weighted) treated obs than control obs",
+                   "no valid matches"))
+
+  if (typeof(m.out) == "logical" && all(is.na(m.out))) {
+    .err("no units were matched")
+  }
 
   #Note: must use character match.matrix because of re-ordering treat into treat_
   mm <- matrix(NA_integer_, nrow = n1, ncol = max(table(m.out$index.treated)),
@@ -460,44 +540,23 @@ matchit2genetic <- function(treat, data, distance, discarded,
     mm[ind1__[i], seq_along(matched.units)] <- matched.units
   }
 
-  # }
-  # else {
-  #   #Use nn_match() instead of Match()
-  #   ord1 <- ord[ord %in% which(treat == 1)]
-  #   if (!is.null(cov.cals)) calcovs <- get.covs.matrix(reformulate(cov.cals), data = data)
-  #   else calcovs <- NULL
-  #
-  #   if (is.null(g.out)) MWM <- generalized_inverse(cov(X))
-  #   else MWM <- g.out$Weight.matrix %*% diag(1/apply(X, 2, var))
-  #
-  #   if (isFALSE(A$fast)) {
-  #     mm <- nn_match(treat, ord1, ratio, replace, discarded, distance, ex, dist.cal,
-  #                    cov.cals, calcovs, X, MWM)
-  #   }
-  #   else {
-  #     mm <- nn_matchC(treat, ord1, ratio, replace, discarded, distance, ex, dist.cal,
-  #                     cov.cals, calcovs, X, MWM)
-  #   }
-  #
-  #   mm[] <- names(treat)[mm]
-  #   dimnames(mm) <- list(lab1, seq_len(ratio))
-  # }
-
-  if (verbose) cat("Calculating matching weights... ")
+  .cat_verbose("Calculating matching weights... ", verbose = verbose)
 
   if (replace) {
     psclass <- NULL
+    weights <- get_weights_from_mm(mm, treat, 1L)
   }
   else {
-    psclass <- mm2subclass(mm, treat)
+    psclass <- mm2subclass(mm, treat, 1L)
+    weights <- get_weights_from_subclass(psclass, treat)
   }
 
   res <- list(match.matrix = nummm2charmm(mm, treat),
               subclass = psclass,
-              weights = get_weights_from_mm(mm, treat),
+              weights = weights,
               obj = g.out)
 
-  if (verbose) cat("Done.\n")
+  .cat_verbose("Done.\n", verbose = verbose)
 
   class(res) <- "matchit"
   res
