@@ -180,15 +180,21 @@
 #'
 #' data("lalonde")
 #' m.out <- matchit(treat ~ age + educ + married +
-#'                    race + re74, data = lalonde,
-#'                  method = "nearest", exact = ~ married,
+#'                    race + re74,
+#'                  data = lalonde,
+#'                  method = "nearest",
+#'                  exact = ~ married,
 #'                  replace = TRUE)
+#'
 #' summary(m.out, interactions = TRUE)
 #'
 #' s.out <- matchit(treat ~ age + educ + married +
 #'                    race + nodegree + re74 + re75,
-#'                  data = lalonde, method = "subclass")
+#'                  data = lalonde,
+#'                  method = "subclass")
+#'
 #' summary(s.out, addlvariables = ~log(age) + I(re74==0))
+#'
 #' summary(s.out, subclass = TRUE)
 #'
 
@@ -204,59 +210,19 @@ summary.matchit <- function(object,
                             ...) {
 
   #Create covariate matrix; include caliper, exact, and mahvars
-
-  if (is.null(object$X)) {
-    X <- matrix(nrow = length(object$treat), ncol = 0)
-  }
-  else {
-    X <- get.covs.matrix(data = object$X)
-  }
-  X_assign <- get_assign(X)
-
-  if (!is.null(addlvariables)) {
-    if (is.character(addlvariables)) {
-      if (!is.null(data) && is.data.frame(data)) {
-        if (all(addlvariables %in% names(data))) {
-          addlvariables <- data[addlvariables]
-        }
-        else {
-          stop("All variables in 'addlvariables' must be in 'data'.", call. = FALSE)
-        }
-      }
-      else {
-        stop("If 'addlvariables' is specified as a string, a data frame argument must be supplied to 'data'.", call. = FALSE)
-      }
-    }
-    else if (inherits(addlvariables, "formula")) {
-      vars.in.formula <- all.vars(addlvariables)
-      if (!is.null(data) && is.data.frame(data)) {
-        data <- data.frame(data[names(data) %in% vars.in.formula],
-                           object$X[names(object$X) %in% setdiff(vars.in.formula, names(data))])
-      }
-      else data <- object$X
-
-      addlvariables <- get.covs.matrix(addlvariables, data = data)
-    }
-    else if (!is.matrix(addlvariables) && !is.data.frame(addlvariables)) {
-      stop("The argument to 'addlvariables' must be in one of the accepted forms. See ?summary.matchit for details.", call. = FALSE)
-    }
-
-    if (is.data.frame(addlvariables)) {
-      addlvariables <- get.covs.matrix(data = addlvariables)
-    }
-
-    addl_assign <- get_assign(addlvariables)
-    X <- cbind(X, addlvariables[, setdiff(colnames(addlvariables), colnames(X)), drop = FALSE])
-  }
+  X <- .process_X(object, addlvariables, data)
 
   treat <- object$treat
   weights <- object$weights
-  s.weights <- if (is.null(object$s.weights)) rep(1, length(weights)) else object$s.weights
+  s.weights <- {
+    if (is_null(object$s.weights)) rep_with(1, weights)
+    else object$s.weights
+  }
 
-  no_x <- length(X) == 0
+  no_x <- is_null(X)
 
   if (no_x) {
-    X <- matrix(1, nrow = length(treat), ncol = 1,
+    X <- matrix(1, nrow = length(treat), ncol = 1L,
                 dimnames = list(names(treat), ".1"))
     nam <- colnames(X)
   }
@@ -270,30 +236,36 @@ summary.matchit <- function(object,
 
   kk <- ncol(X)
 
-  matched <- !is.null(object$info$method)
+  matched <- is_not_null(object$info$method)
   un <- un || !matched
 
-  if (standardize) {
-    s.d.denom <- switch(object$estimand,
-                        "ATT" = "treated",
-                        "ATC" = "control",
-                        "ATE" = "pooled")
+  chk::chk_flag(interactions)
+  chk::chk_flag(standardize)
+  chk::chk_flag(pair.dist)
+  chk::chk_flag(un)
+  chk::chk_flag(improvement)
+
+  s.d.denom <- {
+    if (standardize) switch(object$estimand,
+                            "ATT" = "treated",
+                            "ATC" = "control",
+                            "ATE" = "pooled")
+    else NULL
   }
-  else s.d.denom <- NULL
 
   ## Summary Stats
   if (un) {
-    aa.all <- lapply(seq_len(kk), function(i) bal1var(X[,i], tt = treat, ww = NULL, s.weights = s.weights,
+    aa.all <- lapply(seq_len(kk), function(i) bal1var(X[, i], tt = treat, ww = NULL, s.weights = s.weights,
                                                       standardize = standardize, s.d.denom = s.d.denom))
     sum.all <- do.call("rbind", aa.all)
-    dimnames(sum.all) <- list(nam, names(aa.all[[1]]))
+    dimnames(sum.all) <- list(nam, names(aa.all[[1L]]))
 
-    if (no_x) sum.all <- sum.all[-1, , drop = FALSE]
+    if (no_x) sum.all <- sum.all[-1L, , drop = FALSE]
     sum.all.int <- NULL
   }
 
   if (matched) {
-    aa.matched <- lapply(seq_len(kk), function(i) bal1var(X[,i], tt = treat, ww = weights,
+    aa.matched <- lapply(seq_len(kk), function(i) bal1var(X[, i], tt = treat, ww = weights,
                                                           s.weights = s.weights,
                                                           subclass = object$subclass,
                                                           mm = object$match.matrix,
@@ -301,34 +273,37 @@ summary.matchit <- function(object,
                                                           s.d.denom = s.d.denom,
                                                           compute.pair.dist = pair.dist))
     sum.matched <- do.call("rbind", aa.matched)
-    dimnames(sum.matched) <- list(nam, names(aa.matched[[1]]))
+    dimnames(sum.matched) <- list(nam, names(aa.matched[[1L]]))
 
     if (no_x) sum.matched <- sum.matched[-1, , drop = FALSE]
     sum.matched.int <- NULL
   }
 
   if (!no_x && interactions) {
-    n.int <- kk*(kk+1)/2
-    if (un) sum.all.int <- matrix(NA_real_, nrow = n.int, ncol = length(aa.all[[1]]), dimnames = list(NULL, names(aa.all[[1]])))
-    if (matched) sum.matched.int <- matrix(NA_real_, nrow = n.int, ncol = length(aa.matched[[1]]), dimnames = list(NULL, names(aa.matched[[1]])))
+    n.int <- kk * (kk + 1) / 2
+    if (un) sum.all.int <- matrix(NA_real_, nrow = n.int, ncol = length(aa.all[[1L]]),
+                                  dimnames = list(NULL, names(aa.all[[1]])))
 
-    to.remove <- rep(FALSE, n.int)
+    if (matched) sum.matched.int <- matrix(NA_real_, nrow = n.int, ncol = length(aa.matched[[1L]]),
+                                           dimnames = list(NULL, names(aa.matched[[1L]])))
+
+    to.remove <- rep.int(FALSE, n.int)
     int.names <- character(n.int)
     k <- 1
-    for (i in 1:kk) {
+    for (i in seq_len(kk)) {
       for (j in i:kk) {
-        x2 <- X[,i] * X[,j]
+        x2 <- X[, i] * X[, j]
         if (all(abs(x2) < sqrt(.Machine$double.eps)) ||
-            all(abs(x2 - X[,i]) < sqrt(.Machine$double.eps))) { #prevent interactions within same factors
+            all(abs(x2 - X[, i]) < sqrt(.Machine$double.eps))) { #prevent interactions within same factors
           to.remove[k] <- TRUE
         }
         else {
           if (un) {
-            sum.all.int[k,] <- bal1var(x2, tt = treat, ww = NULL, s.weights = s.weights,
+            sum.all.int[k, ] <- bal1var(x2, tt = treat, ww = NULL, s.weights = s.weights,
                                        standardize = standardize, s.d.denom = s.d.denom)
           }
           if (matched) {
-            sum.matched.int[k,] <- bal1var(x2, tt = treat, ww = weights, s.weights = s.weights,
+            sum.matched.int[k, ] <- bal1var(x2, tt = treat, ww = weights, s.weights = s.weights,
                                            subclass = object$subclass, mm = object$match.matrix,
                                            standardize = standardize, s.d.denom = s.d.denom,
                                            compute.pair.dist = pair.dist)
@@ -338,7 +313,7 @@ summary.matchit <- function(object,
             int.names[k] <- paste0(nam[i], "\u00B2")
           }
           else {
-            int.names[k] <- paste(nam[i], nam[j], sep=" * ")
+            int.names[k] <- paste(nam[i], nam[j], sep = " * ")
           }
         }
         k <- k + 1
@@ -347,36 +322,38 @@ summary.matchit <- function(object,
 
     if (un) {
       rownames(sum.all.int) <- int.names
-      sum.all <- rbind(sum.all, sum.all.int[!to.remove,,drop = FALSE])
+      sum.all <- rbind(sum.all, sum.all.int[!to.remove, , drop = FALSE])
     }
+
     if (matched) {
       rownames(sum.matched.int) <- int.names
-      sum.matched <- rbind(sum.matched, sum.matched.int[!to.remove,,drop = FALSE])
+      sum.matched <- rbind(sum.matched, sum.matched.int[!to.remove, , drop = FALSE])
     }
   }
 
-  if (!is.null(object$distance)) {
+  if (is_not_null(object$distance)) {
     if (un) {
       ad.all <- bal1var(object$distance, tt = treat, ww = NULL, s.weights = s.weights,
                         standardize = standardize, s.d.denom = s.d.denom)
-      if (!exists("sum.all", inherits = FALSE)) {
-        sum.all <- matrix(ad.all, nrow = 1, dimnames = list("distance", names(ad.all)))
+      if (exists("sum.all", inherits = FALSE)) {
+        sum.all <- rbind(ad.all, sum.all)
+        rownames(sum.all)[1L] <- "distance"
       }
       else {
-        sum.all <- rbind(ad.all, sum.all)
-        rownames(sum.all)[1] <- "distance"
+        sum.all <- matrix(ad.all, nrow = 1L, dimnames = list("distance", names(ad.all)))
       }
     }
     if (matched) {
       ad.matched <- bal1var(object$distance, tt = treat, ww = weights, s.weights = s.weights,
                             subclass = object$subclass, mm = object$match.matrix, standardize = standardize,
                             s.d.denom = s.d.denom, compute.pair.dist = pair.dist)
-      if (!exists("sum.matched", inherits = FALSE)) {
-        sum.matched <- matrix(ad.matched, nrow = 1, dimnames = list("distance", names(ad.matched)))
+      if (exists("sum.matched", inherits = FALSE)) {
+        sum.matched <- rbind(ad.matched, sum.matched)
+        rownames(sum.matched)[1L] <- "distance"
       }
       else {
-        sum.matched <- rbind(ad.matched, sum.matched)
-        rownames(sum.matched)[1] <- "distance"
+        sum.matched <- matrix(ad.matched, nrow = 1L,
+                              dimnames = list("distance", names(ad.matched)))
       }
     }
   }
@@ -385,16 +362,16 @@ summary.matchit <- function(object,
   if (matched && un && improvement) {
     reduction <- matrix(NA_real_, nrow = nrow(sum.all), ncol = ncol(sum.all) - 2,
                         dimnames = list(rownames(sum.all), colnames(sum.all)[-(1:2)]))
-    stat.all <- abs(sum.all[,-(1:2), drop = FALSE])
-    stat.matched <- abs(sum.matched[,-(1:2), drop = FALSE])
+    stat.all <- abs(sum.all[, -(1:2), drop = FALSE])
+    stat.matched <- abs(sum.matched[, -(1:2), drop = FALSE])
 
     #Everything but variance ratios
-    reduction[,-2] <- 100*(stat.all[,-2]-stat.matched[,-2])/stat.all[,-2]
+    reduction[, -2] <- 100 * (stat.all[, -2] - stat.matched[, -2]) / stat.all[, -2]
 
     #Just variance ratios; turn to log first
-    vr.all <- abs(log(stat.all[,2]))
-    vr.matched <- abs(log(stat.matched[,2]))
-    reduction[,2] <- 100*(vr.all-vr.matched)/vr.all
+    vr.all <- abs(log(stat.all[, 2]))
+    vr.matched <- abs(log(stat.matched[, 2]))
+    reduction[, 2] <- 100 * (vr.all - vr.matched) / vr.all
 
     reduction[stat.all == 0 & stat.matched == 0] <- 0
     reduction[stat.all == 0 & stat.matched > 0] <- -Inf
@@ -407,10 +384,15 @@ summary.matchit <- function(object,
   nn <- nn(treat, weights, object$discarded, s.weights)
 
   ## output
-  res <- list(call = object$call, nn = nn, sum.all = if (un) sum.all,
-              sum.matched = if (matched) sum.matched, reduction = reduction)
+  res <- list(call = object$call,
+              nn = nn,
+              sum.all = if (un) sum.all,
+              sum.matched = if (matched) sum.matched,
+              reduction = reduction)
+
   class(res) <- "summary.matchit"
-  return(res)
+
+  res
 }
 
 #' @exportS3Method summary matchit.subclass
@@ -427,50 +409,15 @@ summary.matchit.subclass <- function(object,
                                      ...) {
 
   #Create covariate matrix
-  X <- get.covs.matrix(data = object$X)
-
-  if (!is.null(addlvariables)) {
-    if (is.character(addlvariables)) {
-      if (!is.null(data) && is.data.frame(data)) {
-        if (all(addlvariables %in% names(data))) {
-          addlvariables <- data[addlvariables]
-        }
-        else {
-          stop("All variables in 'addlvariables' must be in 'data'.", call. = FALSE)
-        }
-      }
-      else {
-        stop("If 'addlvariables' is specified as a string, a data frame argument must be supplied to 'data'.", call. = FALSE)
-      }
-    }
-    else if (inherits(addlvariables, "formula")) {
-      vars.in.formula <- all.vars(addlvariables)
-      if (!is.null(data) && is.data.frame(data)) data <- data.frame(data[names(data) %in% vars.in.formula],
-                                                                    object$X[names(data) %in% setdiff(vars.in.formula, names(data))])
-      else data <- object$X
-
-      addlvariables <- get.covs.matrix(addlvariables, data = data)
-    }
-    else if (!is.matrix(addlvariables) && !is.data.frame(addlvariables)) {
-      stop("The argument to 'addlvariables' must be in one of the accepted forms. See ?summary.matchit for details.", call. = FALSE)
-    }
-
-    if (is.data.frame(addlvariables)) {
-      if (!all(vapply(addlvariables, is.numeric, logical(1L)))) {
-        addlvariables <- get.covs.matrix(data = addlvariables)
-      }
-      else {
-        addlvariables <- as.matrix(addlvariables)
-      }
-    }
-  }
-
-  X <- cbind(X, addlvariables[, setdiff(colnames(addlvariables), colnames(X)), drop = FALSE])
+  X <- .process_X(object, addlvariables, data)
 
   which.subclass <- subclass
   treat <- object$treat
   weights <- object$weights
-  s.weights <- if (is.null(object$s.weights)) rep(1, length(weights)) else object$s.weights
+  s.weights <- {
+    if (is_null(object$s.weights)) rep_with(1, weights)
+    else object$s.weights
+  }
   subclass <- object$subclass
 
   nam <- colnames(X)
@@ -478,24 +425,35 @@ summary.matchit.subclass <- function(object,
   kk <- ncol(X)
   subclasses <- levels(subclass)
 
-  if (standardize) {
-    s.d.denom <- switch(object$estimand,
-                        "ATT" = "treated",
-                        "ATC" = "control",
-                        "ATE" = "pooled")
-  }
-  else s.d.denom <- NULL
+  chk::chk_flag(interactions)
+  chk::chk_flag(standardize)
+  chk::chk_flag(pair.dist)
+  chk::chk_flag(un)
+  chk::chk_flag(improvement)
 
-  if (isTRUE(which.subclass)) which.subclass <- subclasses
-  else if (isFALSE(which.subclass)) which.subclass <- NULL
-  else if (!is.atomic(which.subclass) || !all(which.subclass %in% seq_along(subclasses))) {
-    stop("'subclass' should be TRUE, FALSE, or a vector of subclass indices for which subclass balance is to be displayed.",
-         call. = FALSE)
+  s.d.denom <- {
+    if (standardize) switch(object$estimand,
+                            "ATT" = "treated",
+                            "ATC" = "control",
+                            "ATE" = "pooled")
+    else NULL
+  }
+
+  if (isTRUE(which.subclass)) {
+    which.subclass <- subclasses
+  }
+  else if (isFALSE(which.subclass)) {
+    which.subclass <- NULL
+  }
+  else if (is.atomic(which.subclass) && all(which.subclass %in% seq_along(subclasses))) {
+    which.subclass <- subclasses[which.subclass]
+  }
+  else {
+    .err("`subclass` should be `TRUE`, `FALSE`, or a vector of subclass indices for which subclass balance is to be displayed")
   }
-  else which.subclass <- subclasses[which.subclass]
 
   matched <- TRUE #always compute aggregate balance so plot.summary can use it
-  subs <- !is.null(which.subclass)
+  subs <- is_not_null(which.subclass)
 
   ## Aggregate Subclass
   #Use the estimated weights to compute aggregate balance.
@@ -504,48 +462,50 @@ summary.matchit.subclass <- function(object,
   sum.all <- sum.matched <- sum.subclass <- reduction <- NULL
 
   if (un) {
-    aa.all <- setNames(lapply(seq_len(kk), function(i) bal1var(X[,i], tt = treat, ww = NULL, s.weights = s.weights,
+    aa.all <- setNames(lapply(seq_len(kk), function(i) bal1var(X[, i], tt = treat, ww = NULL, s.weights = s.weights,
                                                                standardize = standardize, s.d.denom = s.d.denom)),
                        colnames(X))
     sum.all <- do.call("rbind", aa.all)
-    dimnames(sum.all) <- list(nam, names(aa.all[[1]]))
+    dimnames(sum.all) <- list(nam, names(aa.all[[1L]]))
 
     sum.all.int <- NULL
   }
 
   if (matched) {
-    aa.matched <- setNames(lapply(seq_len(kk), function(i) bal1var(X[,i], tt = treat, ww = weights, s.weights = s.weights,
+    aa.matched <- setNames(lapply(seq_len(kk), function(i) bal1var(X[, i], tt = treat, ww = weights, s.weights = s.weights,
                                                                    subclass = subclass, standardize = standardize,
                                                                    s.d.denom = s.d.denom, compute.pair.dist = pair.dist)),
                            colnames(X))
     sum.matched <- do.call("rbind", aa.matched)
-    dimnames(sum.matched) <- list(nam, names(aa.matched[[1]]))
+    dimnames(sum.matched) <- list(nam, names(aa.matched[[1L]]))
 
     sum.matched.int <- NULL
   }
 
   if (interactions) {
-    n.int <- kk*(kk+1)/2
-    if (un) sum.all.int <- matrix(NA_real_, nrow = n.int, ncol = length(aa.all[[1]]), dimnames = list(NULL, names(aa.all[[1]])))
-    if (matched) sum.matched.int <- matrix(NA_real_, nrow = n.int, ncol = length(aa.matched[[1]]), dimnames = list(NULL, names(aa.matched[[1]])))
+    n.int <- kk * (kk + 1) / 2
+    if (un) sum.all.int <- matrix(NA_real_, nrow = n.int, ncol = length(aa.all[[1L]]),
+                                  dimnames = list(NULL, names(aa.all[[1L]])))
+    if (matched) sum.matched.int <- matrix(NA_real_, nrow = n.int, ncol = length(aa.matched[[1L]]),
+                                           dimnames = list(NULL, names(aa.matched[[1L]])))
 
-    to.remove <- rep(FALSE, n.int)
+    to.remove <- rep.int(FALSE, n.int)
     int.names <- character(n.int)
-    k <- 1
-    for (i in 1:kk) {
+    k <- 1L
+    for (i in seq_len(kk)) {
       for (j in i:kk) {
-        x2 <- X[,i] * X[,j]
+        x2 <- X[, i] * X[, j]
         if (all(abs(x2) < sqrt(.Machine$double.eps)) ||
-            all(abs(x2 - X[,i]) < sqrt(.Machine$double.eps))) { #prevent interactions within same factors
+            all(abs(x2 - X[, i]) < sqrt(.Machine$double.eps))) { #prevent interactions within same factors
           to.remove[k] <- TRUE
         }
         else {
           if (un) {
-            sum.all.int[k,] <- bal1var(x2, tt = treat, ww = NULL, s.weights = s.weights,
+            sum.all.int[k, ] <- bal1var(x2, tt = treat, ww = NULL, s.weights = s.weights,
                                        standardize = standardize, s.d.denom = s.d.denom)
           }
           if (matched) {
-            sum.matched.int[k,] <- bal1var(x2, tt = treat, ww = weights, s.weights = s.weights,
+            sum.matched.int[k, ] <- bal1var(x2, tt = treat, ww = weights, s.weights = s.weights,
                                            subclass = subclass, standardize = standardize,
                                            compute.pair.dist = pair.dist)
           }
@@ -553,44 +513,44 @@ summary.matchit.subclass <- function(object,
             int.names[k] <- paste0(nam[i], "\u00B2")
           }
           else {
-            int.names[k] <- paste(nam[i], nam[j], sep=" * ")
+            int.names[k] <- paste(nam[i], nam[j], sep = " * ")
           }
         }
-        k <- k + 1
+        k <- k + 1L
       }
     }
 
     if (un) {
       rownames(sum.all.int) <- int.names
-      sum.all <- rbind(sum.all, sum.all.int[!to.remove,,drop = FALSE])
+      sum.all <- rbind(sum.all, sum.all.int[!to.remove, , drop = FALSE])
     }
     if (matched) {
       rownames(sum.matched.int) <- int.names
-      sum.matched <- rbind(sum.matched, sum.matched.int[!to.remove,,drop = FALSE])
+      sum.matched <- rbind(sum.matched, sum.matched.int[!to.remove, , drop = FALSE])
     }
   }
 
-  if (!is.null(object$distance)) {
+  if (is_not_null(object$distance)) {
     if (un) {
       ad.all <- bal1var(object$distance, tt = treat, ww = NULL, s.weights = s.weights,
                         standardize = standardize, s.d.denom = s.d.denom)
       sum.all <- rbind(ad.all, sum.all)
-      rownames(sum.all)[1] <- "distance"
+      rownames(sum.all)[1L] <- "distance"
     }
     if (matched) {
       ad.matched <- bal1var(object$distance, tt = treat, ww = weights, s.weights = s.weights,
                             subclass = subclass, standardize = standardize,
                             s.d.denom = s.d.denom, compute.pair.dist = pair.dist)
       sum.matched <- rbind(ad.matched, sum.matched)
-      rownames(sum.matched)[1] <- "distance"
+      rownames(sum.matched)[1L] <- "distance"
     }
   }
 
   ## Imbalance Reduction
   if (un && matched && improvement) {
-    stat.all <- abs(sum.all[,-(1:2)])
-    stat.matched <- abs(sum.matched[,-(1:2)])
-    reduction <- 100*(stat.all-stat.matched)/stat.all
+    stat.all <- abs(sum.all[, -(1:2)])
+    stat.matched <- abs(sum.matched[, -(1:2)])
+    reduction <- 100 * (stat.all - stat.matched) / stat.all
 
     reduction[stat.all == 0 & stat.matched == 0] <- 0
     reduction[stat.all == 0 & stat.matched > 0] <- -Inf
@@ -603,26 +563,31 @@ summary.matchit.subclass <- function(object,
       #bal1var.subclass only returns unmatched stats, which is all we need within
       #subclasses. Otherwise, identical to matched stats.
       aa <- setNames(lapply(seq_len(kk), function(i) {
-        bal1var.subclass(X[,i], tt = treat, s.weights = s.weights, subclass = subclass, s.d.denom = s.d.denom, standardize = standardize, which.subclass = s)
+        bal1var.subclass(X[, i], tt = treat, s.weights = s.weights,
+                         subclass = subclass, s.d.denom = s.d.denom,
+                         standardize = standardize, which.subclass = s)
       }), colnames(X))
 
-      sum.sub <- matrix(NA_real_, nrow = kk, ncol = ncol(aa[[1]]), dimnames = list(nam, colnames(aa[[1]])))
+      sum.sub <- matrix(NA_real_, nrow = kk, ncol = ncol(aa[[1L]]), dimnames = list(nam, colnames(aa[[1L]])))
 
       sum.sub.int <- NULL
-      for (i in 1:kk) {
-        sum.sub[i,] <- aa[[i]]
+      for (i in seq_len(kk)) {
+        sum.sub[i, ] <- aa[[i]]
       }
       if (interactions) {
-        sum.sub.int <- matrix(NA_real_, nrow = kk*(kk+1)/2, ncol = length(aa[[1]]), dimnames = list(NULL, names(aa[[1]])))
-        to.remove <- rep(FALSE, nrow(sum.sub.int))
+        sum.sub.int <- matrix(NA_real_, nrow = kk * (kk + 1) / 2, ncol = length(aa[[1L]]),
+                              dimnames = list(NULL, names(aa[[1L]])))
+        to.remove <- rep.int(FALSE, nrow(sum.sub.int))
         int.names <- character(nrow(sum.sub.int))
-        k <- 1
-        for (i in 1:kk) {
+        k <- 1L
+        for (i in seq_len(kk)) {
           for (j in i:kk) {
             if (!to.remove[k]) { #to.remove defined above
-              x2 <- X[,i] * X[,j]
-              jqoi <- bal1var.subclass(x2, tt = treat, s.weights = s.weights, subclass = subclass, s.d.denom = s.d.denom, standardize = standardize, which.subclass = s)
-              sum.sub.int[k,] <- jqoi
+              x2 <- X[, i] * X[, j]
+              jqoi <- bal1var.subclass(x2, tt = treat, s.weights = s.weights,
+                                       subclass = subclass, s.d.denom = s.d.denom,
+                                       standardize = standardize, which.subclass = s)
+              sum.sub.int[k, ] <- jqoi
               if (i == j) {
                 int.names[k] <- paste0(nam[i], "\u00B2")
               }
@@ -635,17 +600,17 @@ summary.matchit.subclass <- function(object,
         }
         rownames(sum.sub.int) <- int.names
 
-        sum.sub <- rbind(sum.sub, sum.sub.int[!to.remove,,drop = FALSE])
+        sum.sub <- rbind(sum.sub, sum.sub.int[!to.remove, , drop = FALSE])
       }
 
-      if (!is.null(object$distance)) {
+      if (is_not_null(object$distance)) {
         ad <- bal1var.subclass(object$distance, tt = treat, s.weights = s.weights, subclass = subclass,
                                s.d.denom = s.d.denom, standardize = standardize, which.subclass = s)
         sum.sub <- rbind(ad, sum.sub)
-        rownames(sum.sub)[1] <- "distance"
+        rownames(sum.sub)[1L] <- "distance"
       }
 
-      return(sum.sub)
+      sum.sub
     })
     names(sum.subclass) <- paste("Subclass", which.subclass)
   }
@@ -654,62 +619,56 @@ summary.matchit.subclass <- function(object,
   qn <- qn(treat, subclass, object$discarded)
   nn <- nn(treat, weights, object$discarded, s.weights)
 
-  # if (subs) {
-  #   small.subclass.control <- which.subclass[qn["Control", as.character(which.subclass)] <= 1]
-  #   if (length(small.subclass.control) > 0) {
-  #     if (length(small.subclass.control) == 1) warning(paste0("Not enough control units in subclass ", small.subclass.control, "."), call.= FALSE)
-  #     else warning(paste0("Not enough control units in subclasses ", word_list(small.subclass.control), "."), call. = FALSE)
-  #   }
-  #
-  #   small.subclass.treated <- which.subclass[qn["Treated", as.character(which.subclass)] <= 1]
-  #   if (length(small.subclass.treated) > 0) {
-  #     if (length(small.subclass.treated) == 1) warning(paste0("Not enough treated units in subclass ", small.subclass.treated, "."), call.= FALSE)
-  #     else warning(paste0("Not enough treated units in subclasses ", word_list(small.subclass.treated), "."), call. = FALSE)
-  #   }
-  # }
-
   ## output
-  res <- list(call = object$call, sum.all = sum.all, sum.across = sum.matched,
-              sum.subclass = sum.subclass, reduction = reduction,
-              qn = qn, nn = nn)
+  res <- list(call = object$call,
+              sum.all = sum.all,
+              sum.across = sum.matched,
+              sum.subclass = sum.subclass,
+              reduction = reduction,
+              qn = qn,
+              nn = nn)
+
   class(res) <- c("summary.matchit.subclass", "summary.matchit")
-  return(res)
+
+  res
 }
 
 #' @exportS3Method print summary.matchit
 #' @rdname summary.matchit
 print.summary.matchit <- function(x, digits = max(3, getOption("digits") - 3),
-                                  ...){
+                                  ...) {
 
-  if (!is.null(x$call)) cat("\nCall:", deparse(x$call), sep = "\n")
+  if (is_not_null(x$call)) {
+    cat("\nCall:", deparse(x$call), sep = "\n")
+  }
 
-  if (!is.null(x$sum.all)) {
+  if (is_not_null(x$sum.all)) {
     cat("\nSummary of Balance for All Data:\n")
-    print(round_df_char(x$sum.all[,-7, drop = FALSE], digits, pad = "0", na_vals = "."),
+    print(round_df_char(x$sum.all[, -7, drop = FALSE], digits, pad = "0", na_vals = "."),
           right = TRUE, quote = FALSE)
   }
 
-  if (!is.null(x$sum.matched)) {
+  if (is_not_null(x$sum.matched)) {
     cat("\nSummary of Balance for Matched Data:\n")
-    if (all(is.na(x$sum.matched[,7]))) x$sum.matched <- x$sum.matched[,-7,drop = FALSE] #Remove pair dist if empty
+    if (all(is.na(x$sum.matched[, 7]))) x$sum.matched <- x$sum.matched[, -7, drop = FALSE] #Remove pair dist if empty
     print(round_df_char(x$sum.matched, digits, pad = "0", na_vals = "."),
           right = TRUE, quote = FALSE)
   }
-  if (!is.null(x$reduction)) {
+  if (is_not_null(x$reduction)) {
     cat("\nPercent Balance Improvement:\n")
-    print(round_df_char(x$reduction[,-5, drop = FALSE], 1, pad = "0", na_vals = "."), right = TRUE,
+    print(round_df_char(x$reduction[, -5, drop = FALSE], 1, pad = "0", na_vals = "."), right = TRUE,
           quote = FALSE)
   }
-  if (!is.null(x$nn)) {
+  if (is_not_null(x$nn)) {
     cat("\nSample Sizes:\n")
     nn <- x$nn
-    if (isTRUE(all.equal(nn["All (ESS)",], nn["All",]))) {
+    if (isTRUE(all.equal(nn["All (ESS)", ], nn["All", ]))) {
       #Don't print ESS if same as full SS
-      nn <- nn[rownames(nn) != "All (ESS)",,drop = FALSE]
+      nn <- nn[rownames(nn) != "All (ESS)", , drop = FALSE]
     }
-    if (isTRUE(all.equal(nn["Matched (ESS)",], nn["Matched",]))) {
+    if (isTRUE(all.equal(nn["Matched (ESS)", ], nn["Matched", ]))) {
       #Don't print ESS if same as matched SS
-      nn <- nn[rownames(nn) != "Matched (ESS)",,drop = FALSE]
+      nn <- nn[rownames(nn) != "Matched (ESS)", , drop = FALSE]
     }
     print(round_df_char(nn, 2, pad = " ", na_vals = "."), right = TRUE,
           quote = FALSE)
@@ -719,52 +678,54 @@ print.summary.matchit <- function(x, digits = max(3, getOption("digits") - 3),
 }
 
 #' @exportS3Method print summary.matchit.subclass
-print.summary.matchit.subclass <- function(x, digits = max(3, getOption("digits") -  3), ...){
+print.summary.matchit.subclass <- function(x, digits = max(3L, getOption("digits") -  3L), ...) {
 
-  if (!is.null(x$call)) cat("\nCall:", deparse(x$call), sep = "\n")
+  if (is_not_null(x$call)) {
+    cat("\nCall:", deparse(x$call), sep = "\n")
+  }
 
-  if (!is.null(x$sum.all)) {
+  if (is_not_null(x$sum.all)) {
     cat("\nSummary of Balance for All Data:\n")
-    print(round_df_char(x$sum.all[,-7, drop = FALSE], digits, pad = "0", na_vals = "."),
+    print(round_df_char(x$sum.all[, -7L, drop = FALSE], digits, pad = "0", na_vals = "."),
           right = TRUE, quote = FALSE)
   }
 
-  if (length(x$sum.subclass) > 0) {
+  if (is_not_null(x$sum.subclass)) {
     cat("\nSummary of Balance by Subclass:\n")
     for (s in seq_along(x$sum.subclass)) {
       cat(paste0("\n- ", names(x$sum.subclass)[s], "\n"))
-      print(round_df_char(x$sum.subclass[[s]][,-7, drop = FALSE], digits, pad = "0", na_vals = "."),
+      print(round_df_char(x$sum.subclass[[s]][, -7L, drop = FALSE], digits, pad = "0", na_vals = "."),
             right = TRUE, quote = FALSE)
     }
-    if (!is.null(x$qn)) {
+    if (is_not_null(x$qn)) {
       cat("\nSample Sizes by Subclass:\n")
       print(round_df_char(x$qn, 2, pad = " ", na_vals = "."),
             right = TRUE, quote = FALSE)
     }
   }
   else {
-    if (!is.null(x$sum.across)) {
+    if (is_not_null(x$sum.across)) {
       cat("\nSummary of Balance Across Subclasses\n")
-      if (all(is.na(x$sum.across[,7]))) x$sum.across <- x$sum.across[,-7,drop = FALSE]
+      if (all(is.na(x$sum.across[, 7L]))) x$sum.across <- x$sum.across[, -7L, drop = FALSE]
       print(round_df_char(x$sum.across, digits, pad = "0", na_vals = "."),
             right = TRUE, quote = FALSE)
     }
-    if (!is.null(x$reduction)) {
+    if (is_not_null(x$reduction)) {
       cat("\nPercent Balance Improvement:\n")
-      print(round_df_char(x$reduction[,-5, drop = FALSE], 1, pad = "0", na_vals = "."),
+      print(round_df_char(x$reduction[, -5L, drop = FALSE], 1L, pad = "0", na_vals = "."),
             right = TRUE, quote = FALSE)
     }
 
-    if (!is.null(x$nn)) {
+    if (is_not_null(x$nn)) {
       cat("\nSample Sizes:\n")
       nn <- x$nn
-      if (isTRUE(all.equal(nn["All (ESS)",], nn["All",]))) {
+      if (isTRUE(all.equal(nn["All (ESS)", ], nn["All", ]))) {
         #Don't print ESS if same as full SS
-        nn <- nn[rownames(nn) != "All (ESS)",,drop = FALSE]
+        nn <- nn[rownames(nn) != "All (ESS)", , drop = FALSE]
       }
-      if (isTRUE(all.equal(nn["Matched (ESS)",], nn["Matched",]))) {
+      if (isTRUE(all.equal(nn["Matched (ESS)", ], nn["Matched", ]))) {
         #Don't print ESS if same as matched SS
-        nn <- nn[rownames(nn) != "Matched (ESS)",,drop = FALSE]
+        nn <- nn[rownames(nn) != "Matched (ESS)", , drop = FALSE]
       }
       print(round_df_char(nn, 2, pad = " ", na_vals = "."),
             right = TRUE, quote = FALSE)
@@ -772,3 +733,99 @@ print.summary.matchit.subclass <- function(x, digits = max(3, getOption("digits"
   }
   cat("\n")
 }
+
+.process_X <- function(object, addlvariables = NULL, data = NULL) {
+
+  X <- {
+    if (is_null(object$X)) matrix(nrow = length(object$treat), ncol = 0)
+    else get_covs_matrix(data = object$X)
+  }
+
+  if (is_null(addlvariables)) {
+    return(X)
+  }
+
+  #Attempt to extract data from matchit object; same as match_data()
+  data.found <- FALSE
+  for (i in 1:4) {
+    if (i == 2L) {
+      data <- try(eval(object$call$data, envir = environment(object$formula)), silent = TRUE)
+    }
+    else if (i == 3L) {
+      data <- try(eval(object$call$data, envir = parent.frame()), silent = TRUE)
+    }
+    else if (i == 4L) {
+      data <- object[["model"]][["data"]]
+    }
+
+    if (!null_or_error(data) && length(dim(data)) == 2L && nrow(data) == length(object[["treat"]])) {
+      data.found <- TRUE
+      break
+    }
+  }
+
+  if (is.character(addlvariables)) {
+    if (is_null(data) || !is.data.frame(data)) {
+      .err("if `addlvariables` is specified as a string, a data frame argument must be supplied to `data`")
+    }
+
+    if (!all(hasName(data, addlvariables))) {
+      .err("all variables in `addlvariables` must be in `data`")
+    }
+
+    addlvariables <- data[addlvariables]
+  }
+  else if (rlang::is_formula(addlvariables)) {
+    if (is_not_null(data) && is.data.frame(data)) {
+      vars.in.formula <- all.vars(addlvariables)
+      data <- cbind(data[names(data) %in% vars.in.formula],
+                    object$X[names(object$X) %in% setdiff(vars.in.formula, names(data))])
+    }
+    else {
+      data <- object$X
+    }
+  }
+  else if (!is.matrix(addlvariables) && !is.data.frame(addlvariables)) {
+    .err("the argument to `addlvariables` must be in one of the accepted forms. See `?summary.matchit` for details")
+  }
+
+  af <- rlang::is_formula(addlvariables)
+  if (af) {
+    addvariables_f <- addlvariables
+    addlvariables <- model.frame(addvariables_f, data = data, na.action = "na.pass")
+  }
+
+  if (nrow(addlvariables) != length(object$treat)) {
+    if (is_null(data) || data.found) {
+      .err("variables specified in `addlvariables` must have the same number of units as are present in the original call to `matchit()`")
+    }
+    else {
+      .err("`data` must have the same number of units as are present in the original call to `matchit()`")
+    }
+  }
+
+  k <- ncol(addlvariables)
+  for (i in seq_len(k)) {
+    if (anyNA(addlvariables[[i]]) || (is.numeric(addlvariables[[i]]) && !all(is.finite(addlvariables[[i]])))) {
+      covariates.with.missingness <- names(addlvariables)[i:k][vapply(i:k, function(j) anyNA(addlvariables[[j]]) ||
+                                                                        (is.numeric(addlvariables[[j]]) &&
+                                                                           !all(is.finite(addlvariables[[j]]))),
+                                                                      logical(1L))]
+      .err(paste0("Missing and non-finite values are not allowed in `addlvariables`. Variables with missingness or non-finite values:\n\t",
+                  toString(covariates.with.missingness)), tidy = FALSE)
+    }
+
+    if (is.character(addlvariables[[i]])) {
+      addlvariables[[i]] <- factor(addlvariables[[i]])
+    }
+  }
+
+  addlvariables <- {
+    if (af) get_covs_matrix(addvariables_f, data = data)
+    else get_covs_matrix(data = addlvariables)
+  }
+
+  # addl_assign <- get_assign(addlvariables)
+  cbind(X, addlvariables[, setdiff(colnames(addlvariables), colnames(X)), drop = FALSE])
+
+}
